# revert.py
#
# Copyright (C) 2014 Tim Marston <tim@edm.am>
#
# This file is part of stdhome (hereafter referred to as "this program").
# See http://ed.am/dev/stdhome for more information.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import sys, re, getopt
from command import Command
import stdhome.the as the
from stdhome.deployment import Deployment
from stdhome.walker.copy_out import CopyOutWalker


class RevertCommand( Command ):


	def __init__( self ):
		self.all = False
		self.quiet = False


	def print_help( self ):
		print "Usage: " + the.program.name + " revert [--repo=REPO] [--all] [FILE]..."
		print
		#      01234567890123456789012345678901234567890123456789012345678901234567890123456789
		print "Revert any local changes to a file in the home directory."
		print
		print "This reverts the content of the named files, in the local filesystem, specified"
		print "relative to the home directory, to its state in the repo.  Specifying individual"
		print "files and directories will revert them (recursively, in the case of"
		print "directories).  Or you can revert all files in your home directory by specifying"
		print "no files and explicitly using the --all option."
		print
		print "To see a list files that can be reverted, type:"
		print "    " + the.program.name + " status"
		print
		print "Options:"
		print "      --all        confirm that you want to revert all files"
		print "      --quiet      do not report changes to the home directory"
		print "  -r, --repo=REPO  select the repo to check-out or create (defaults to 'home')"
		print "  -v, --verbose    display information about what is being done"
		print "      --help       display help and exit"
		exit( 0 )


	def parse_command_line( self ):
		opts, args = getopt.gnu_getopt(
			sys.argv[ 1: ], "r:v",
			[ "all", "quiet", "repo=", "verbose", "help" ] )
		for opt, optarg in opts:
			if opt == "--all":
				self.all = True
			elif opt == "--quiet":
				self.quiet = True
			elif opt in [ '--repo', '-r' ]:
				if not re.search( '^[-a-zA-z0-9.]+$', optarg ):
					raise the.program.FatalError(
						'invalid repository name: ' + optarg )
				the.repo = optarg
			elif opt == "--help":
				self.print_help()

		# discard first argument (the command)
		args.pop( 0 )

		# remaining arguments
		self.files = args

		# check that either files or --all was given
		if len( self.files ) == 0 and not self.all:
			raise the.program.UsageError(
				"no files were specified and --all was not given" )
		if len( self.files ) > 0 and self.all:
			raise the.program.UsageError(
				"files specified, which conflicts with using --all" )


	def run( self ):

		# set up repo and check it exists
		the.repo.check_dir_exists()

		# determine files
		files = self.expand_files( self.files )

		# initialise deployment and ensure it's not ongoing
		deployment = Deployment()
		deployment.check_ongoing( False )

		# check for local changes
		if the.repo.vcs.has_changes():
			raise the.program.FatalError(
				'repo has local changes: %s\n'
				'Hint: see "%s stage-revert --help"' %
				( the.repo.name, the.program.name ) )

		# check status
		walker = CopyOutWalker( files if files else None, self.quiet )
		walker.walk()
